@@ -1,12 +1,12 @@
 #!/bin/sh
 #
-# WWWOFFLE - World Wide Web Offline Explorer - Version 2.8.
+# WWWOFFLE - World Wide Web Offline Explorer - Version 2.9.
 #
-# A Perl script to update the configuration file to version 2.8 standard (from version 2.7).
+# A Perl script to update the configuration file to version 2.9 standard (from version 2.8).
 #
 # Written by Andrew M. Bishop
 #
-# This file Copyright 2000,01,02,03 Andrew M. Bishop
+# This file Copyright 2000,01,02,03,04,05,06 Andrew M. Bishop
 # It may be distributed under the GNU Public License, version 2, or
 # any higher version.  See section COPYING of the GNU Public license
 # for conditions under which this file may be redistributed.
@@ -22,69 +22,55 @@ $#ARGV==0 || die "Usage: $0 wwwoffle.conf\n";
 
 $conf=$ARGV[0];
 
-$version="2.8";
+$version="2.9";
 
-$urlspec="[^ \t\r\n:<!]+://[^ \t\r\n/=]+/?[^ \t\r\n=>]*";
-$urlspec1="([^ \t\r\n:<!]+)://([^ \t\r\n/=]+)(/?[^ \t\r\n=>]*)";
+$urlspec="[^ \t:<!]+://[^ \t/=]+/?[^ \t=>]*";
+$urlspec1="([^ \t:<!]+)://([^ \t/=]+)(/?[^ \t=>]*)";
 
-# The new options that have been added (since version 2.7).
+# The new options that have been added (since version 2.8).
 
-%new_Options=(
-              "reply-chunked-data *=" , "reply-chunked-data = yes"
+%new_StartUp=(
+              "https-port *=" , "https-port = 8443"
              );
 
-%new_OnlineOptions=(
-                    "pragma-no-cache *="        , "pragma-no-cache = yes",
-                    "cache-control-no-cache *=" , "cache-control-no-cache = yes",
-                    "request-conditional *="    , "request-conditional = yes",
-                    "validate-with-etag *="     , "validate-with-etag = yes",
-                    "request-chunked-data *="   , "request-chunked-data = yes"
-                   );
-
-%new_FetchOptions=(
-                   "icon-images *=" , "icon-images = no"
-                  );
-
-%new_CensorHeader=(
-                   "force-user-agent *=" , "force-user-agent = no"
-                  );
-
-%new_ModifyHTML=(
-                 "disable-meta-set-cookie *=" , "disable-meta-set-cookie = no"
-                  );
-
-%new_options=(
-              "Options"        , \%new_Options,
-              "OnlineOptions"  , \%new_OnlineOptions,
-              "FetchOptions"   , \%new_FetchOptions,
-              "CensorHeader"   , \%new_CensorHeader,
-              "ModifyHTML"     , \%new_ModifyHTML
-              );
-
-# The options that have changed (since version 2.7).
-
-%changed_CensorHeader=(
-                       "^#? *(<$urlspec>) *User-Agent *= *WWWOFFLE/[0-9.]+[a-z]*(.*)", " \$1 User-Agent = WWWOFFLE/$version\$2\n",
-                       "^#? *User-Agent *= *WWWOFFLE/[0-9.]+[a-z]*(.*)"              , " User-Agent = WWWOFFLE/$version\$1\n",
-                       );
-
-%changed_options=(
-                  "CensorHeader"   , \%changed_CensorHeader
-                  );
-
-# The options that have been moved (since version 2.7).
-
-%moved_options=(
+%new_SSLOptions=(
+                 "enable-caching *="  , "enable-caching = no",
+                 "allow-tunnel *="    , "#allow-tunnel = *:443"
                 );
 
-# The options that have been deleted (since version 2.7).
+%new_MIMETypes=(
+                ".pem *="  , ".pem     = application/x-x509-ca-cert"
+               );
 
-@deleted_ModifyHTML=(
-                     "enable-modify-online *="
+%new_options=(
+              "StartUp"        , \%new_StartUp,
+              "SSLOptions"     , \%new_SSLOptions,
+              "MIMETypes"      , \%new_MIMETypes
+              );
+
+# The options that have changed (since version 2.8).
+
+%changed_SSLOptions=(
+                     "ssl-allow-port *= *(.+)\$" , " allow-tunnel = *:\$1"
                     );
 
+%changed_options=(
+                  "SSLOptions"   , \%changed_SSLOptions
+                  );
+
+# The options that have been moved (since version 2.8).
+
+%moved_Options=(
+                "ssl-allow-port *=" , "SSLOptions"
+               );
+
+%moved_options=(
+                "Options"        , \%moved_Options
+                );
+
+# The options that have been deleted (since version 2.8).
+
 %deleted_options=(
-                  "ModifyHTML" , deleted_ModifyHTML
                   );
 
 # The sections in the configuration file.
@@ -94,6 +80,7 @@ $urlspec1="([^ \t\r\n:<!]+)://([^ \t\r\n/=]+)(/?[^ \t\r\n=>]*)";
            "Options"            ,
            "OnlineOptions"      ,
            "OfflineOptions"     ,
+           "SSLOptions"         ,
            "FetchOptions"       ,
            "IndexOptions"       ,
            "ModifyHTML"         ,
@@ -119,6 +106,7 @@ $urlspec1="([^ \t\r\n:<!]+)://([^ \t\r\n/=]+)(/?[^ \t\r\n=>]*)";
           "Options"            , \@Options,
           "OnlineOptions"      , \@OnlineOptions,
           "OfflineOptions"     , \@OfflineOptions,
+          "SSLOptions"         , \@SSLOptions,
           "FetchOptions"       , \@FetchOptions,
           "IndexOptions"       , \@IndexOptions,
           "ModifyHTML"         , \@ModifyHTML,
@@ -152,13 +140,17 @@ if(open(CURR,"<$conf"))
    while(<CURR>)
        {
         $lineno++;
-        if(m/^\# WWWOFFLE - World Wide Web Offline Explorer - (Version 2\.[0-69][a-z]?)/)
+        s/\r*\n/\n/;
+
+        if(m/^\# WWWOFFLE - World Wide Web Offline Explorer - (Version 2\.[0-7][a-z]?)/)
             {
-             die "\nExisting configuration file is not for version 2.7 or 2.8.\n".
+             die "\nExisting configuration file is not for version 2.8 or 2.9.\n".
                  "(The header line says that it is '$1')\n".
-                 "Try running upgrade-config-2.x-2.5.pl\n".
+                 "Try running upgrade-config-2.0-2.5.pl\n".
                  "     and/or upgrade-config-2.5-2.6.pl\n".
-                 "     and/or upgrade-config-2.6-2.7.pl.\n\n";
+                 "     and/or upgrade-config-2.6-2.7.pl\n".
+                 "     and/or upgrade-config-2.7-2.8.pl\n".
+                 "\n";
             }
         next if(/^[ \t]*\#/);
         next if(/^[ \t]*$/);
@@ -168,6 +160,7 @@ if(open(CURR,"<$conf"))
              while(<CURR>)
                  {
                   $lineno++;
+                  s/\r*\n/\n/;
                   last if(/^[ \t]*\}/);
 
                   push(@{$options{$section}},$_);
@@ -190,9 +183,9 @@ if(open(CURR,"<$conf"))
              while(<CURR>)
                  {
                   $lineno++;
-                  s/\r*\n//;
+                  s/\r*\n/\n/;
                   s/\t/    /;
-                  $incconf=$1,last if(m/^[ \t]*([^ \t\#]+)[ \t]*$/);
+                  $incconf=$1,last if(m/^[ \t]*([^ \t\n\#]+)[ \t]*$/);
                  }
 
              @conf=split('/',$conf);
@@ -205,6 +198,8 @@ if(open(CURR,"<$conf"))
 
              while(<INC>)
                  {
+                  s/\r*\n/\n/;
+
                   push(@{$options{$section}},$_);
 
                   # Warn about URL-SPECIFICATION changes: http://www.foo/ is not http://www.foo/*
@@ -223,6 +218,7 @@ if(open(CURR,"<$conf"))
              while(<CURR>)
                  {
                   $lineno++;
+                  s/\r*\n/\n/;
                   last if(/^[ \t]*\]/);
                  }
 
@@ -230,7 +226,7 @@ if(open(CURR,"<$conf"))
              next;
             }
 
-        if(!$section && /^[ \t]*([a-zA-Z]+)[ \t\r\n]*$/)
+        if(!$section && /^[ \t]*([a-zA-Z]+)[ \t]*$/)
             {
              $section=$1;
              next;
@@ -256,7 +252,9 @@ if(open(INST,"<wwwoffle.conf.install") || open(INST,"<$conf.install"))
 
    while(<INST>)
        {
-        if(!$section && (/^[ \t]*[\#{]/ || /^[ \t\r\n]*$/))
+        s/\r*\n/\n/;
+
+        if(!$section && (/^[ \t]*[\#{]/ || /^[ \t]*$/))
              {
               $comment.=$_;
               next;
@@ -268,7 +266,7 @@ if(open(INST,"<wwwoffle.conf.install") || open(INST,"<$conf.install"))
              next;
             }
 
-        if(!$section && /^[ \t]*([a-zA-Z]+)[ \t\r\n]*$/)
+        if(!$section && /^[ \t]*([a-zA-Z]+)[ \t]*$/)
             {
              $section=$1;
              $comments{$section}=$comment;
@@ -331,7 +329,7 @@ foreach $section (keys(%moved_options))
              if($line =~ m/$regexp_option/)
                  {
                   $new_section=$ {$moved_options{$section}}{$regexp_option};
-                  $old_line=$line; $old_line =~ s/ *\r*\n//g; $old_line =~ s/^ *//;
+                  $old_line=$line; $old_line =~ s/[ \t]*\n//g; $old_line =~ s/^[ \t]*//;
                   print "$section\t- Moved option '$old_line' -> $new_section\n";
 
                   if(!defined $first{$new_section})
@@ -359,7 +357,7 @@ foreach $section (keys(%changed_options))
             {
              if($line =~ m/$regexp_option/)
                  {
-                  $old_line=$line; $old_line =~ s/ *\r*\n//g; $old_line =~ s/^ *//;
+                  $old_line=$line; $old_line =~ s/[ \t]*\n//g; $old_line =~ s/^[ \t]*//;
                   $line =~ m/$regexp_option/;
                   $line=$ {$changed_options{$section}}{$regexp_option};
                   $one=$1;
@@ -369,7 +367,8 @@ foreach $section (keys(%changed_options))
                   $line =~ s/\$2/$two/g;
                   $line =~ s/\$3/$three/g;
                   $line="#".$line if($old_line =~ /^\#/);
-                  $new_line=$line; $new_line =~ s/ *\r*\n//g; $new_line =~ s/^ *//;
+                  $line.="\n";
+                  $new_line=$line; $new_line =~ s/[ \t]*\n$//g; $new_line =~ s/^[ \t]*//;
                   print "$section\t- Changed option '$old_line' -> '$new_line'\n" if($old_line ne $new_line);
                  }
             }
@@ -388,7 +387,7 @@ foreach $section (keys(%deleted_options))
             {
              if($line =~ m/$regexp_option/ && $line !~ m/^\#\#\#/)
                  {
-                  $old_line=$line; $old_line =~ s/ *\r*\n//g; $old_line =~ s/^ *//;
+                  $old_line=$line; $old_line =~ s/[ \t]*\n//g; $old_line =~ s/^[ \t]*//;
                   print "$section\t- Deleted option '$old_line'\n";
 
                   if(!defined $first{$section})
