@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/src/RCS/configmisc.c 1.9 2002/03/09 11:45:49 amb Exp $
+  $Header: /home/amb/wwwoffle/src/RCS/configmisc.c 1.32 2006/01/07 16:10:38 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.7a.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.9.
   Configuration file data management functions.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1997,98,99,2000,01,02 Andrew M. Bishop
+  This file Copyright 1997,98,99,2000,01,02,03,04,05,06 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -20,24 +20,19 @@
 #include <string.h>
 #include <ctype.h>
 
-#include <limits.h>
-#include <unistd.h>
-#include <sys/types.h>
 #include <pwd.h>
 #include <grp.h>
 
-#include "configpriv.h"
-#include "errors.h"
-#include "wwwoffle.h"
 #include "misc.h"
+#include "errors.h"
+#include "configpriv.h"
 
 
 /* Local functions */
 
-static void FreeKeyOrValue(KeyOrValue *keyval,ConfigType type);
-
-static char* sprintf_key_or_value(ConfigType type,KeyOrValue key_or_val);
-static char* sprintf_url_spec(UrlSpec *urlspec);
+static /*@null@*/ char* sprintf_key_or_value(ConfigType type,KeyOrValue key_or_val);
+static /*@null@*/ char* sprintf_url_spec(const UrlSpec *urlspec);
+static /*@null@*/ char *strstrn(const char *phaystack, const char *pneedle, size_t n, int nocase);
 
 
 /*+ The backup version of the config file. +*/
@@ -67,20 +62,12 @@ void DefaultConfigFile(void)
           (*item)->val=NULL;
           (*item)->def_val=(KeyOrValue*)malloc(sizeof(KeyOrValue));
 
-#if CONFIG_VERIFY_ABORT
           if(CurrentConfig.sections[s]->itemdefs[i].key_type!=Fixed)
              PrintMessage(Fatal,"Configuration file error at %s:%d",__FILE__,__LINE__);
 
           if((errmsg=ParseKeyOrValue(CurrentConfig.sections[s]->itemdefs[i].def_val,CurrentConfig.sections[s]->itemdefs[i].val_type,(*item)->def_val)))
              PrintMessage(Fatal,"Configuration file error at %s:%d; %s",__FILE__,__LINE__,errmsg);
-#else
-          ParseKeyOrValue(CurrentConfig.sections[s]->itemdefs[i].def_val,CurrentConfig.sections[s]->itemdefs[i].val_type,(*item)->def_val);
-#endif
          }
-
-#if CONFIG_DEBUG_DUMP
- DumpConfigFile();
-#endif
 }
 
 
@@ -156,34 +143,22 @@ void RestoreBackupConfigFile(void)
 
 void PurgeBackupConfigFile(int restore_startup)
 {
- int s=0,i;
+ int s,i;
 
- if(restore_startup)
-   {
-    /* Restore the StartUp section. */
+ /* Purge all of the sections and restore StartUp if needed. */
 
-    for(i=0;i<CurrentConfig.sections[s]->nitemdefs;i++)
-      {
-       FreeConfigItem(*CurrentConfig.sections[s]->itemdefs[i].item);
-
-       *CurrentConfig.sections[s]->itemdefs[i].item=*BackupConfig.sections[s]->itemdefs[i].item;
-
-       free(BackupConfig.sections[s]->itemdefs[i].item);
-      }
-
-    free(BackupConfig.sections[s]->itemdefs);
-    free(BackupConfig.sections[s]);
-
-    s++;
-   }
-
- /* Purge all of the sections except StartUp. */
-
- for(s=1;s<BackupConfig.nsections;s++)
+ for(s=0;s<BackupConfig.nsections;s++)
    {
     for(i=0;i<BackupConfig.sections[s]->nitemdefs;i++)
       {
-       FreeConfigItem(*BackupConfig.sections[s]->itemdefs[i].item);
+       if(s==0 && restore_startup)
+         {
+          FreeConfigItem(*CurrentConfig.sections[s]->itemdefs[i].item);
+
+          *CurrentConfig.sections[s]->itemdefs[i].item=*BackupConfig.sections[s]->itemdefs[i].item;
+         }
+       else
+          FreeConfigItem(*BackupConfig.sections[s]->itemdefs[i].item);
 
        free(BackupConfig.sections[s]->itemdefs[i].item);
       }
@@ -267,7 +242,7 @@ void FreeConfigItem(ConfigItem item)
   ConfigType type The type of key or value.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void FreeKeyOrValue(KeyOrValue *keyval,ConfigType type)
+void FreeKeyOrValue(KeyOrValue *keyval,ConfigType type)
 {
  switch(type)
    {
@@ -300,12 +275,15 @@ static void FreeKeyOrValue(KeyOrValue *keyval,ConfigType type)
    case PathName:
    case FileExt:
    case MIMEType:
-   case HostOrNone:
    case Host:
-   case HostAndPortOrNone:
+   case HostOrNone:
    case HostAndPort:
+   case HostAndPortOrNone:
+   case HostWild:
+   case HostAndPortWild:
    case UserPass:
    case Url:
+   case UrlWild:
     if(keyval->string)
        free(keyval->string);
     break;
@@ -322,29 +300,29 @@ static void FreeKeyOrValue(KeyOrValue *keyval,ConfigType type)
 
   int MatchUrlSpecification Return the matching length if true else 0.
 
-  UrlSpec *spec The URL-SPECIFICATION.
+  const UrlSpec *spec The URL-SPECIFICATION.
 
-  char *proto The protocol.
+  const char *proto The protocol.
 
-  char *host The host.
+  const char *host The host.
 
-  char *path The path.
+  int port The port number (or -1).
 
-  char *args The arguments.
+  const char *path The path.
+
+  const char *args The arguments.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int MatchUrlSpecification(UrlSpec *spec,char *proto,char *host,char *path,char *args)
+int MatchUrlSpecification(const UrlSpec *spec,const char *proto,const char *host,int port,const char *path,const char *args)
 {
- char *hoststr,*portstr;
  int match=0;
 
- SplitHostPort(host,&hoststr,&portstr);
-
  if((!spec->proto || !strcmp(UrlSpecProto(spec),proto)) &&
-    (!spec->host || WildcardMatch(hoststr,UrlSpecHost(spec))) &&
-    (spec->port==-1 || (!portstr && spec->port==0) || (portstr && atoi(portstr)==spec->port)) &&
-    (!spec->path || WildcardMatch(path,UrlSpecPath(spec))) &&
-    (!spec->args || (args && WildcardMatch(args,UrlSpecArgs(spec))) || (!args && *UrlSpecArgs(spec)==0)))
+    (!spec->host || WildcardMatch(host,UrlSpecHost(spec),0)) &&
+    (spec->port==-1 || (port==-1 && spec->port==0) || (port!=-1 && port==spec->port)) &&
+    (!spec->path || WildcardMatch(path,UrlSpecPath(spec),spec->nocase) || 
+     ((!strncmp(UrlSpecPath(spec),"/*/",(size_t)3) && WildcardMatch(path,UrlSpecPath(spec)+2,spec->nocase)))) &&
+    (!spec->args || (args && WildcardMatch(args,UrlSpecArgs(spec),spec->nocase)) || (!args && *UrlSpecArgs(spec)==0)))
    {
     match=(spec->proto?strlen(UrlSpecProto(spec)):0)+
           (spec->host ?strlen(UrlSpecHost(spec) ):0)+
@@ -352,8 +330,6 @@ int MatchUrlSpecification(UrlSpec *spec,char *proto,char *host,char *path,char *
           (spec->args ?strlen(UrlSpecArgs(spec) ):0)+1;
    }
 
- RejoinHostPort(host,hoststr,portstr);
-
  return(match);
 }
 
@@ -363,85 +339,55 @@ int MatchUrlSpecification(UrlSpec *spec,char *proto,char *host,char *path,char *
 
   int WildcardMatch returns 1 if there is a match.
 
-  char *string The fixed string that is being matched.
+  const char *string The fixed string that is being matched.
 
-  char *pattern The pattern to match against.
+  const char *pattern The pattern to match against.
+
+  int nocase A flag that if set to 1 ignores the case of the string.
+
+  By Paul A. Rombouts <p.a.rombouts@home.nl>, handles more than two '*' using simpler algorithm than previously.
+
+  See also the strstrn() function at the bottom of this file.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int WildcardMatch(char *string,char *pattern)
+int WildcardMatch(const char *string,const char *pattern,int nocase)
 {
- int len_beg;
- char *midstr, *endstr;
- char *pattstr, *starp=strchr(pattern,'*');
+ size_t len_beg;
+ const char *midstr, *endstr;
+ const char *pattstr, *starp=strchr(pattern,'*');
 
  if(!starp)
-    return(!strcmp(string,pattern));
+    return(( nocase && !strcasecmp(string,pattern)) ||
+           (!nocase && !strcmp(string,pattern)));
 
  len_beg=starp-pattern;
- if(strncmp(string,pattern,len_beg))
+ if(( nocase && strncasecmp(string,pattern,len_beg)) ||
+    (!nocase && strncmp(string,pattern,len_beg)))
     return(0);
 
  midstr=string+len_beg;
 
  while(pattstr=starp+1,starp=strchr(pattstr,'*'))
    {
-    char *match;
-
-    *starp=0;
-    match=strstr(midstr,pattstr);
-    *starp='*';
+    size_t len_patt=starp-pattstr;
+    char *match=strstrn(midstr,pattstr,len_patt,nocase);
 
     if(!match)
        return(0);
 
-    midstr=match+(starp-pattstr);
+    midstr=match+len_patt;
    }
 
- endstr= strchr(midstr,0)-strlen(pattstr);
+ endstr=midstr+strlen(midstr)-strlen(pattstr);
+
  if(midstr>endstr)
     return(0);
 
- return(!strcmp(endstr,pattstr));
+ return(( nocase && !strcasecmp(endstr,pattstr)) ||
+        (!nocase && !strcmp(endstr,pattstr)));
 }
 
 
-#if CONFIG_DEBUG_DUMP
-
-/*++++++++++++++++++++++++++++++++++++++
-  Remove the old values if the re-read of the file succeeded.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-void DumpConfigFile(void)
-{
- int s,i,e;
-
- fprintf(stderr,"CONFIGURATION FILE\n");
-
- for(s=0;s<CurrentConfig.nsections;s++)
-   {
-    fprintf(stderr,"  Section %s\n",CurrentConfig.sections[s]->name);
-
-    for(i=0;i<CurrentConfig.sections[s]->nitemdefs;i++)
-      {
-       if(*CurrentConfig.sections[s]->itemdefs[i].name)
-          fprintf(stderr,"    Item %s\n",CurrentConfig.sections[s]->itemdefs[i].name);
-       else
-          fprintf(stderr,"    Item [default]\n");
-
-       if(*CurrentConfig.sections[s]->itemdefs[i].item)
-          for(e=0;e<(*CurrentConfig.sections[s]->itemdefs[i].item)->nentries;e++)
-            {
-             char *string=ConfigEntryString(*CurrentConfig.sections[s]->itemdefs[i].item,e,NULL,NULL,NULL);
-             fprintf(stderr,"      %s\n",string);
-             free(string);
-            }
-      }
-   }
-}
-
-#endif
-
-
 /*++++++++++++++++++++++++++++++++++++++
   Return the string that represents the Configuration type.
 
@@ -500,16 +446,24 @@ char *ConfigTypeString(ConfigType type)
     return "HostAndPort";
    case HostAndPortOrNone:
     return "HostAndPortOrNone";            /* val */
+   case HostWild:
+    return "HostWild";                     /* val */
+   case HostAndPortWild:
+    return "HostAndPortWild";              /* val */
    case UserPass:
     return "UserPass";           /* key */
    case Url:
     return "Url";                          /* val */
+   case UrlWild:
+    return "UrlWild";                      /* val */
    case UrlSpecification:
     return "UrlSpecification";   /* key */ /* val */
    }
 
- return(NULL);
-};
+ /*@notreached@*/
+
+ return("??Unknown??");
+}
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -527,19 +481,9 @@ char *ConfigEntryString(ConfigItem item,int which)
  char *url=NULL,*key=NULL,*val=NULL;
  char *string;
 
- /* Handle the URL */
+ /* Get the sub-strings */
 
- if(item->url && item->url[which])
-    url=sprintf_url_spec(item->url[which]);
-
- /* Handle the key */
-
- key=sprintf_key_or_value(item->itemdef->key_type,item->key[which]);
-
- /* Handle the value */
-
- if(item->itemdef->val_type!=None)
-    val=sprintf_key_or_value(item->itemdef->val_type,item->val[which]);
+ ConfigEntryStrings(item,which,&url,&key,&val);
 
  /* Create the string */
 
@@ -547,28 +491,68 @@ char *ConfigEntryString(ConfigItem item,int which)
 
  /* Send the results back */
 
+ if(url) free(url);
+ if(key) free(key);
+ if(val) free(val);
+
  return(string);
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Convert a Configuration entry into a canonical printable string.
+
+  ConfigItem item The configuration item.
+
+  int which Which particular entry in the ConfigItem to print.
+
+  char **url Returns the URL string.
+
+  char **key Returns the key string.
+
+  char **val Returns the value string.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void ConfigEntryStrings(ConfigItem item,int which,char **url,char **key,char **val)
+{
+ /* Handle the URL */
+
+ if(item->url && item->url[which])
+    *url=sprintf_url_spec(item->url[which]);
+ else
+    *url=NULL;
+
+ /* Handle the key */
+
+ *key=sprintf_key_or_value(item->itemdef->key_type,item->key[which]);
+
+ /* Handle the value */
+
+ if(item->itemdef->val_type!=None)
+    *val=sprintf_key_or_value(item->itemdef->val_type,item->val[which]);
+ else
+    *val=NULL;
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Make a Configuration entry string from supplied arguments.
 
   char *MakeConfigEntryString Returns a malloced string.
 
-  ConfigItemDef *itemdef The configuration item definition.
+  const ConfigItemDef *itemdef The configuration item definition.
 
-  char *url Specifies the URL string.
+  const char *url Specifies the URL string.
 
-  char *key Specifies the key string.
+  const char *key Specifies the key string.
 
-  char *val Specifies the val string.
+  const char *val Specifies the val string.
   ++++++++++++++++++++++++++++++++++++++*/
 
-char *MakeConfigEntryString(ConfigItemDef *itemdef,char *url,char *key,char *val)
+char *MakeConfigEntryString(const ConfigItemDef *itemdef,const char *url,const char *key,const char *val)
 {
  int strpos=0;
- char *string=(char*)malloc(8);
+ char *string=(char*)calloc((size_t)8,sizeof(char));
 
  /* Handle the URL */
 
@@ -639,7 +623,7 @@ static char* sprintf_key_or_value(ConfigType type,KeyOrValue key_or_val)
     /* Integer */
 
    case Boolean:
-    string=(char*)malloc(1+3);
+    string=(char*)malloc((size_t)4);
     strcpy(string,key_or_val.integer?"yes":"no");
     break;
 
@@ -649,12 +633,12 @@ static char* sprintf_key_or_value(ConfigType type,KeyOrValue key_or_val)
    case CacheSize:
    case FileSize:
    case Percentage:
-    string=(char*)malloc(1+16);
+    string=(char*)malloc((size_t)(MAX_INT_STR+1));
     sprintf(string,"%d",key_or_val.integer);
     break;
 
    case CfgLogLevel:
-    string=(char*)malloc(1+16);
+    string=(char*)malloc((size_t)17);
     if(key_or_val.integer==Debug)
        strcpy(string,"debug");
     if(key_or_val.integer==Inform)
@@ -669,7 +653,7 @@ static char* sprintf_key_or_value(ConfigType type,KeyOrValue key_or_val)
 
    case UserId:
     {
-     struct passwd *pwd=getpwuid(key_or_val.integer);
+     struct passwd *pwd=getpwuid((uid_t)key_or_val.integer);
      if(pwd)
        {
         string=(char*)malloc(1+strlen(pwd->pw_name));
@@ -677,7 +661,7 @@ static char* sprintf_key_or_value(ConfigType type,KeyOrValue key_or_val)
        }
      else
        {
-        string=(char*)malloc(1+16);
+        string=(char*)malloc((size_t)(MAX_INT_STR+1));
         sprintf(string,"%d",key_or_val.integer);
        }
     }
@@ -685,7 +669,7 @@ static char* sprintf_key_or_value(ConfigType type,KeyOrValue key_or_val)
 
    case GroupId:
     {
-     struct group *grp=getgrgid(key_or_val.integer);
+     struct group *grp=getgrgid((gid_t)key_or_val.integer);
      if(grp)
        {
         string=(char*)malloc(1+strlen(grp->gr_name));
@@ -693,22 +677,22 @@ static char* sprintf_key_or_value(ConfigType type,KeyOrValue key_or_val)
        }
      else
        {
-        string=(char*)malloc(1+16);
+        string=(char*)malloc((size_t)(MAX_INT_STR+1));
         sprintf(string,"%d",key_or_val.integer);
        }
     }
    break;
 
    case FileMode:
-    string=(char*)malloc(1+16);
-    sprintf(string,"0%o",key_or_val.integer);
+    string=(char*)malloc((size_t)(1+MAX_INT_STR+1));
+    sprintf(string,"0%o",(unsigned)key_or_val.integer);
     break;
 
    case AgeDays:
     {
      int weeks,months,years,days=key_or_val.integer;
 
-     string=(char*)malloc(1+16);
+     string=(char*)malloc((size_t)(MAX_INT_STR+1+1));
 
      years=days/365;
      if(years*365==days)
@@ -734,7 +718,7 @@ static char* sprintf_key_or_value(ConfigType type,KeyOrValue key_or_val)
     {
      int weeks,days,hours,minutes,seconds=key_or_val.integer;
 
-     string=(char*)malloc(1+16);
+     string=(char*)malloc((size_t)(MAX_INT_STR+1+1));
 
      weeks=seconds/(3600*24*7);
      if(weeks*(3600*24*7)==seconds)
@@ -768,12 +752,15 @@ static char* sprintf_key_or_value(ConfigType type,KeyOrValue key_or_val)
    case PathName:
    case FileExt:
    case MIMEType:
-   case HostOrNone:
    case Host:
-   case HostAndPortOrNone:
+   case HostOrNone:
    case HostAndPort:
+   case HostAndPortOrNone:
+   case HostWild:
+   case HostAndPortWild:
    case UserPass:
    case Url:
+   case UrlWild:
     if(key_or_val.string)
       {
        string=(char*)malloc(1+strlen(key_or_val.string));
@@ -781,7 +768,7 @@ static char* sprintf_key_or_value(ConfigType type,KeyOrValue key_or_val)
       }
     else
       {
-       string=(char*)malloc(1);
+       string=(char*)malloc((size_t)1);
        *string=0;
       }
     break;
@@ -801,25 +788,27 @@ static char* sprintf_key_or_value(ConfigType type,KeyOrValue key_or_val)
 
   char* sprintf_url_spec Return the new string.
 
-  UrlSpec urlspec The URL-SPECIFICATION to convert.
+  const UrlSpec *urlspec The URL-SPECIFICATION to convert.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static char* sprintf_url_spec(UrlSpec *urlspec)
+static char* sprintf_url_spec(const UrlSpec *urlspec)
 {
  char *string=NULL;
  int strpos=0;
- int newlen;
+ size_t newlen;
 
  if(!urlspec)
     return(NULL);
 
- newlen=1+16;
- if(urlspec->proto) newlen+=strlen(UrlSpecProto(urlspec));
- if(urlspec->host ) newlen+=strlen(UrlSpecHost (urlspec));
- if(urlspec->path ) newlen+=strlen(UrlSpecPath (urlspec));
- if(urlspec->args ) newlen+=strlen(UrlSpecArgs (urlspec));
+ newlen=1+8;
+ if(HasUrlSpecProto(urlspec)) newlen+=strlen(UrlSpecProto(urlspec));
+ if(HasUrlSpecHost (urlspec)) newlen+=strlen(UrlSpecHost (urlspec));
+ if(UrlSpecPort(urlspec)>0)   newlen+=MAX_INT_STR;
+ if(HasUrlSpecPath (urlspec)) newlen+=strlen(UrlSpecPath (urlspec));
+ if(HasUrlSpecArgs (urlspec)) newlen+=strlen(UrlSpecArgs (urlspec));
 
  string=(char*)malloc(newlen);
+ *string=0;
 
  if(urlspec->negated)
    {
@@ -827,22 +816,28 @@ static char* sprintf_url_spec(UrlSpec *urlspec)
     strpos+=strlen(string+strpos);
    }
 
- sprintf(string+strpos,"%s://",urlspec->proto?UrlSpecProto(urlspec):"*");
+ if(urlspec->nocase)
+   {
+    sprintf(string+strpos,"%s","~");
+    strpos+=strlen(string+strpos);
+   }
+
+ sprintf(string+strpos,"%s://",HasUrlSpecProto(urlspec)?UrlSpecProto(urlspec):"*");
  strpos+=strlen(string+strpos);
 
- sprintf(string+strpos,"%s",urlspec->host?UrlSpecHost(urlspec):"*");
+ sprintf(string+strpos,"%s",HasUrlSpecHost(urlspec)?UrlSpecHost(urlspec):"*");
  strpos+=strlen(string+strpos);
 
- if(urlspec->port==0)
+ if(UrlSpecPort(urlspec)==0)
     sprintf(string+strpos,":");
- else if(urlspec->port!=-1)
-    sprintf(string+strpos,":%d",urlspec->port);
+ else if(UrlSpecPort(urlspec)!=-1)
+    sprintf(string+strpos,":%d",UrlSpecPort(urlspec));
  strpos+=strlen(string+strpos);
 
- sprintf(string+strpos,"%s",urlspec->path?UrlSpecPath(urlspec):"/*");
+ sprintf(string+strpos,"%s",HasUrlSpecPath(urlspec)?UrlSpecPath(urlspec):"/*");
  strpos+=strlen(string+strpos);
 
- if(urlspec->args)
+ if(HasUrlSpecArgs(urlspec))
    {
     sprintf(string+strpos,"?%s",*UrlSpecArgs(urlspec)?UrlSpecArgs(urlspec):"*");
     strpos+=strlen(string+strpos);
@@ -850,3 +845,159 @@ static char* sprintf_url_spec(UrlSpec *urlspec)
 
  return(string);
 }
+
+
+
+/* Return the offset of one string within another.
+   Copyright (C) 1994, 1996, 1997, 2000 Free Software Foundation, Inc.
+
+   The GNU C Library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2.1 of the License, or (at your option) any later version.
+
+   The GNU C Library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with the GNU C Library; if not, write to the Free
+   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+   02111-1307 USA.  */
+
+/*
+ * My personal strstr() implementation that beats most other algorithms.
+ * Until someone tells me otherwise, I assume that this is the
+ * fastest implementation of strstr() in C.
+ * I deliberately chose not to comment it.  You should have at least
+ * as much fun trying to understand it, as I had to write it :-).
+ *
+ * Stephen R. van den Berg, berg@pool.informatik.rwth-aachen.de	*/
+
+/* strstrn() is a variation of strstr() that only tries to find
+   the first n characters of "needle" in "haystack".
+   If strlen(needle) happens to be less than n, strstrn() behaves 
+   exactly like strstr().
+   Modifications made by Paul Rombouts <p.a.rombouts@home.nl>.
+*/
+
+/* Added a parameter "nocase" to select case insensitive test (value 1)
+   or case sensitive (0).
+   Modification by Marc Boucher.
+*/
+
+typedef unsigned chartype;
+
+static char *strstrn(const char *phaystack, const char *pneedle, size_t n, int nocase)
+{
+  register const unsigned char *haystack, *needle;
+  register chartype b, c;
+  const unsigned char *needle_end;
+
+  char *lowhaystack=NULL;
+  char *lowneedle=NULL;
+
+  if (nocase) {
+     int i;
+     lowhaystack=malloc(strlen(phaystack)+1);
+     lowneedle=malloc(strlen(phaystack)+1);
+
+     for(i=0;phaystack[i];i++) { lowhaystack[i]=tolower(phaystack[i]); }
+     lowhaystack[i]=0;
+     for(i=0;pneedle[i];i++) { lowneedle[i]=tolower(pneedle[i]); }
+     lowneedle[i]=0;
+     
+     phaystack=lowhaystack;
+     pneedle=lowneedle;
+     }
+
+  haystack = (const unsigned char *) phaystack;
+  needle = (const unsigned char *) pneedle;
+  needle_end = needle+n;
+
+  if (needle != needle_end && (b = *needle) != '\0' )
+    {
+      haystack--;                               /* possible ANSI violation */
+      do
+        {
+          c = *++haystack;
+          if (c == '\0')
+            goto ret0;
+        }
+      while (c != b);
+
+      if (++needle == needle_end || (c = *needle) == '\0')
+        goto foundneedle;
+      ++needle;
+      goto jin;
+
+      for (;;)
+        {
+          register chartype a;
+          register const unsigned char *rhaystack, *rneedle;
+
+          do
+            {
+              a = *++haystack;
+              if (a == '\0')
+                goto ret0;
+              if (a == b)
+                break;
+              a = *++haystack;
+              if (a == '\0')
+                goto ret0;
+shloop:
+              ;
+            }
+          while (a != b);
+
+jin:      a = *++haystack;
+          if (a == '\0')
+            goto ret0;
+
+          if (a != c)
+            goto shloop;
+
+          rhaystack = haystack-- + 1;
+          if(needle == needle_end) goto foundneedle;
+          rneedle = needle;
+          a = *rneedle;
+
+          if (*rhaystack == a)
+            do
+              {
+                if (a == '\0')
+                  goto foundneedle;
+                ++rhaystack;
+                if(++needle == needle_end) goto foundneedle;
+                a = *needle;
+                if (*rhaystack != a)
+                  break;
+                if (a == '\0')
+                  goto foundneedle;
+                ++rhaystack;
+                if(++needle == needle_end) goto foundneedle;
+                a = *needle;
+              }
+            while (*rhaystack == a);
+
+          needle = rneedle;             /* took the register-poor approach */
+
+          if (a == '\0')
+            break;
+        }
+    }
+foundneedle:
+  if (nocase) {
+     if (lowhaystack) free(lowhaystack);
+     if (lowneedle) free(lowneedle);
+     }
+  return (char*) haystack;
+ret0:
+  if (nocase) {
+     if (lowhaystack) free(lowhaystack);
+     if (lowneedle) free(lowneedle);
+     }
+  return 0;
+}
