@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/src/RCS/refresh.c 2.76 2003/10/19 13:27:08 amb Exp $
+  $Header: /home/amb/wwwoffle/src/RCS/refresh.c 2.89 2005/12/10 15:11:31 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.8.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.9.
   The HTML interactive page to refresh a URL.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1997,98,99,2000,01,02,03 Andrew M. Bishop
+  This file Copyright 1997,98,99,2000,01,02,03,04,05 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -22,30 +22,48 @@
 #include "wwwoffle.h"
 #include "io.h"
 #include "misc.h"
+#include "proto.h"
 #include "errors.h"
 #include "config.h"
 #include "document.h"
 
 
+/*+ The maximum depth of recursion for following Location headers. +*/
 #define MAX_RECURSE_LOCATION 8
 
-/*+ The options for recursive or normal fetching. +*/
-static char *recurse_url=NULL,*recurse_limit="";
-static int recurse_depth=-1,recurse_force=0;
-static int recurse_stylesheets=0,recurse_images=0,recurse_frames=0,recurse_scripts=0,recurse_objects=0;
-static int recurse_location=0;
 
-static void ParseRecurseOptions(char *options);
+/* Local variables */
 
-static /*@null@*/ char *RefreshFormParse(int fd,/*@null@*/ char *request_args,/*@null@*/ Body *request_body);
+/*+ The options for recursive fetching; +*/
+static char *recurse_url=NULL,  /*+ the starting URL. +*/
+            *recurse_limit="";  /*+ the the URL prefix that must match. +*/
 
-static int request_url(URL *Url,char *refresh,URL *refUrl);
+/*+ The options for the recursive fetching; +*/
+static int recurse_depth=-1,    /*+ the depth to go from this URL. +*/
+           recurse_force=0;     /*+ the option to force refreshing. +*/
+
+/*+ The options for the recursive fetching of related items; +*/
+static int recurse_stylesheets=0, /*+ stylesheets. +*/
+           recurse_images=0,      /*+ images. +*/
+           recurse_frames=0,      /*+ frames. +*/
+           recurse_scripts=0,     /*+ scripts. +*/
+           recurse_objects=0,     /*+ objects. +*/
+           recurse_location=0;    /*+ Location headers. +*/
+
+
+/* Local functions */
+
+static void ParseRecurseOptions(URL *Url);
+
+static /*@null@*/ char *RefreshFormParse(int fd,URL *Url,/*@null@*/ char *request_args,/*@null@*/ Body *request_body);
+
+static int request_url(URL *Url,/*@null@*/ char *refresh,URL *refUrl);
 
 
 /*++++++++++++++++++++++++++++++++++++++
   Send to the client a page to allow refreshes using HTML.
 
-  char *RefreshPage Returns a modified URLs for a simple refresh.
+  URL *RefreshPage Returns a modified URL for a simple refresh.
 
   int fd The file descriptor of the client.
 
@@ -56,17 +74,17 @@ static int request_url(URL *Url,char *refresh,URL *refUrl);
   int *recurse Return value set to true if a recursive fetch was asked for.
   ++++++++++++++++++++++++++++++++++++++*/
 
-char *RefreshPage(int fd,URL *Url,Body *request_body,int *recurse)
+URL *RefreshPage(int fd,URL *Url,Body *request_body,int *recurse)
 {
- char *newurl=NULL;
- char *url;
-
- if(Url->args)
-    url=URLDecodeFormArgs(Url->args);
- else
-    url=NULL;
+ URL *newUrl=NULL;
 
  if(!strcmp("/refresh-options/",Url->path))
+   {
+    char *url=NULL;
+
+    if(Url->args)
+       url=URLDecodeFormArgs(Url->args);
+
     HTMLMessage(fd,200,"WWWOFFLE Refresh Form",NULL,"RefreshPage",
                 "url",url,
                 "stylesheets",ConfigBooleanURL(FetchStyleSheets,NULL)?"yes":NULL,
@@ -75,32 +93,39 @@ char *RefreshPage(int fd,URL *Url,Body *request_body,int *recurse)
                 "scripts",ConfigBooleanURL(FetchScripts,NULL)?"yes":NULL,
                 "objects",ConfigBooleanURL(FetchObjects,NULL)?"yes":NULL,
                 NULL);
+
+    if(url)
+       free(url);
+   }
  else if(!strcmp("/refresh-request/",Url->path) && Url->args)
    {
-    if((newurl=RefreshFormParse(fd,Url->args,request_body)))
+    char *newurl;
+    newurl=RefreshFormParse(fd,Url,Url->args,request_body);
+    if(newurl)
+      {
+       newUrl=SplitURL(newurl);
        *recurse=-1;
+       free(newurl);
+      }
    }
- else if(!strcmp("/refresh/",Url->path) && url)
+ else if(!strcmp("/refresh/",Url->path) && Url->args)
    {
-    newurl=(char*)malloc(strlen(url)+1);
-    strcpy(newurl,url);
+    char *url=URLDecodeFormArgs(Url->args);
+    newUrl=SplitURL(url);
+    free(url);
    }
  else if(!strcmp("/refresh-recurse/",Url->path) && Url->args)
    {
     *recurse=1;
-    ParseRecurseOptions(Url->args);
-    newurl=(char*)malloc(strlen(recurse_url)+1);
-    strcpy(newurl,recurse_url);
+    ParseRecurseOptions(Url);
+    newUrl=SplitURL(recurse_url);
    }
  else
     HTMLMessage(fd,404,"WWWOFFLE Illegal Refresh Page",NULL,"RefreshIllegal",
                 "url",Url->pathp,
                 NULL);
 
- if(url)
-    free(url);
-
- return(newurl);
+ return(newUrl);
 }
 
 
@@ -111,18 +136,20 @@ char *RefreshPage(int fd,URL *Url,Body *request_body,int *recurse)
 
   int fd The file descriptor of the client.
 
+  URL *Url The URL of the form that is being processed.
+
   char *request_args The arguments of the requesting URL.
 
   Body *request_body The body of the HTTP request sent by the client.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static char *RefreshFormParse(int fd,char *request_args,Body *request_body)
+static char *RefreshFormParse(int fd,URL *Url,char *request_args,Body *request_body)
 {
  int i;
  char **args,*url=NULL,*limit="",*method=NULL;
  int depth=0,force=0;
  int stylesheets=0,images=0,frames=0,scripts=0,objects=0;
- URL *Url;
+ URL *refUrl;
  char *new_url;
 
  if(!request_args && !request_body)
@@ -144,24 +171,26 @@ static char *RefreshFormParse(int fd,char *request_args,Body *request_body)
 
  for(i=0;args[i];i++)
    {
-    if(!strncmp("url=",args[i],4))
+    if(!strncmp("url=",args[i],(size_t)4))
        url=TrimArgs(URLDecodeFormArgs(args[i]+4));
-    if(!strncmp("depth=",args[i],6))
+    else if(!strncmp("depth=",args[i],(size_t)6))
        depth=atoi(args[i]+6);
-    if(!strncmp("method=",args[i],7))
+    else if(!strncmp("method=",args[i],(size_t)7))
        method=args[i]+7;
-    if(!strncmp("force=",args[i],6))
+    else if(!strncmp("force=",args[i],(size_t)6))
        force=!!(args[i][6]=='Y');
-    if(!strncmp("stylesheets=",args[i],12))
+    else if(!strncmp("stylesheets=",args[i],(size_t)12))
        stylesheets=!!(args[i][12]=='Y');
-    if(!strncmp("images=",args[i],7))
+    else if(!strncmp("images=",args[i],(size_t)7))
        images=!!(args[i][7]=='Y');
-    if(!strncmp("frames=",args[i],7))
+    else if(!strncmp("frames=",args[i],(size_t)7))
        frames=!!(args[i][7]=='Y');
-    if(!strncmp("scripts=",args[i],8))
+    else if(!strncmp("scripts=",args[i],(size_t)8))
        scripts=!!(args[i][8]=='Y');
-    if(!strncmp("objects=",args[i],8))
+    else if(!strncmp("objects=",args[i],(size_t)8))
        objects=!!(args[i][8]=='Y');
+    else
+       PrintMessage(Warning,"Unexpected argument '%s' seen decoding form data for URL '%s'.",args[i],Url->name);
    }
 
  if(url==NULL || *url==0 || method==NULL || *method==0)
@@ -170,29 +199,31 @@ static char *RefreshFormParse(int fd,char *request_args,Body *request_body)
                 "body",request_body?request_body->content:request_args,
                 NULL);
     if(url) free(url);
+    free(args[0]);
+    free(args);
     return(NULL);
    }
 
- Url=SplitURL(url);
+ refUrl=SplitURL(url);
  free(url);
 
  if(!strcmp(method,"any"))
     limit="";
  else if(!strcmp(method,"proto"))
    {
-    limit=(char*)malloc(strlen(Url->proto)+4);
-    sprintf(limit,"%s://",Url->proto);
+    limit=(char*)malloc(strlen(refUrl->proto)+4);
+    sprintf(limit,"%s://",refUrl->proto);
    }
  else if(!strcmp(method,"host"))
    {
-    limit=(char*)malloc(strlen(Url->proto)+strlen(Url->host)+5);
-    sprintf(limit,"%s://%s/",Url->proto,Url->host);
+    limit=(char*)malloc(strlen(refUrl->proto)+strlen(refUrl->hostport)+5);
+    sprintf(limit,"%s://%s/",refUrl->proto,refUrl->hostport);
    }
  else if(!strcmp(method,"dir"))
    {
     char *p;
-    limit=(char*)malloc(strlen(Url->proto)+strlen(Url->host)+strlen(Url->path)+5);
-    sprintf(limit,"%s://%s%s",Url->proto,Url->host,Url->path);
+    limit=(char*)malloc(strlen(refUrl->proto)+strlen(refUrl->hostport)+strlen(refUrl->path)+5);
+    sprintf(limit,"%s://%s%s",refUrl->proto,refUrl->hostport,refUrl->path);
     p=limit+strlen(limit)-1;
     while(p>limit && *p!='/')
        *p--=0;
@@ -203,12 +234,12 @@ static char *RefreshFormParse(int fd,char *request_args,Body *request_body)
  free(args[0]);
  free(args);
 
- new_url=CreateRefreshPath(Url,limit,depth,force,stylesheets,images,frames,scripts,objects);
+ new_url=CreateRefreshPath(refUrl,limit,depth,force,stylesheets,images,frames,scripts,objects);
 
  if(*limit)
     free(limit);
 
- FreeURL(Url);
+ FreeURL(refUrl);
 
  return(new_url);
 }
@@ -217,16 +248,16 @@ static char *RefreshFormParse(int fd,char *request_args,Body *request_body)
 /*++++++++++++++++++++++++++++++++++++++
   Parse the recursive url options to decide what needs fetching recursively.
 
-  char *options The options to use, encoding the depth and other things.
+  URL *Url The URL containing the options to use, encoding the depth and other things.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void ParseRecurseOptions(char *options)
+static void ParseRecurseOptions(URL *Url)
 {
  recurse_url=NULL;
  recurse_depth=-1;
  recurse_limit="";
 
- if(options)
+ if(Url->args)
    {
     char **args;
     int i;
@@ -237,29 +268,30 @@ static void ParseRecurseOptions(char *options)
     recurse_scripts=0;
     recurse_objects=0;
 
-    args=SplitFormArgs(options);
+    args=SplitFormArgs(Url->args);
 
     for(i=0;args[i];i++)
       {
-       if(!strncmp("url=",args[i],4))
+       if(!strncmp("url=",args[i],(size_t)4))
           recurse_url=TrimArgs(URLDecodeFormArgs(args[i]+4));
-       if(!strncmp("depth=",args[i],6))
+       else if(!strncmp("depth=",args[i],(size_t)6))
          {recurse_depth=atoi(args[i]+6); if(recurse_depth<0) recurse_depth=0;}
-       if(!strncmp("limit=",args[i],6))
+       else if(!strncmp("limit=",args[i],(size_t)6))
           recurse_limit=TrimArgs(URLDecodeFormArgs(args[i]+6));
-       if(!strncmp("force=",args[i],6))
+       else if(!strncmp("force=",args[i],(size_t)6))
           recurse_force=!!(args[i][6]=='Y');
-
-       if(!strncmp("stylesheets=",args[i],12))
+       else if(!strncmp("stylesheets=",args[i],(size_t)12))
           recurse_stylesheets=!!(args[i][12]=='Y')*2;
-       if(!strncmp("images=",args[i],7))
+       else if(!strncmp("images=",args[i],(size_t)7))
           recurse_images=!!(args[i][7]=='Y')*2;
-       if(!strncmp("frames=",args[i],7))
+       else if(!strncmp("frames=",args[i],(size_t)7))
           recurse_frames=!!(args[i][7]=='Y')*2;
-       if(!strncmp("scripts=",args[i],8))
+       else if(!strncmp("scripts=",args[i],(size_t)8))
           recurse_scripts=!!(args[i][8]=='Y')*2;
-       if(!strncmp("objects=",args[i],8))
+       else if(!strncmp("objects=",args[i],(size_t)8))
           recurse_objects=!!(args[i][8]=='Y')*2;
+       else
+          PrintMessage(Warning,"Unexpected argument '%s' seen decoding form data for URL '%s'.",args[i],Url->name);
       }
 
     PrintMessage(Debug,"Recursive Fetch options: depth=%d limit='%s' force=%d",
@@ -338,7 +370,7 @@ int RefreshForced(void)
 
 int RecurseFetch(URL *Url)
 {
- char **list,*metarefresh;
+ URL **list,*metarefresh;
  int more=0,old_recurse_location;
  int j;
 
@@ -346,13 +378,13 @@ int RecurseFetch(URL *Url)
 
  /* A Meta-Refresh header. */
 
- if(recurse_location && (metarefresh=MetaRefresh()))
+ if(recurse_location && (metarefresh=GetReference(RefMetaRefresh)))
    {
-    URL *metarefreshUrl=SplitURL(metarefresh);
+    URL *metarefreshUrl=metarefresh;
 
     recurse_location--;
 
-    if(!metarefreshUrl->local && metarefreshUrl->Protocol)
+    if(!IsLocalHost(metarefreshUrl) && IsProtocolHandled(metarefreshUrl))
       {
        char *refresh=NULL;
 
@@ -364,9 +396,9 @@ int RecurseFetch(URL *Url)
 
        PrintMessage(Debug,"Meta-Refresh=%s",refresh?refresh:metarefreshUrl->name);
        more+=request_url(metarefreshUrl,refresh,Url);
+       if(refresh)
+          free(refresh);
       }
-
-    FreeURL(metarefreshUrl);
    }
 
  recurse_location=MAX_RECURSE_LOCATION;
@@ -376,19 +408,17 @@ int RecurseFetch(URL *Url)
  if(recurse_stylesheets && (list=GetReferences(RefStyleSheet)))
     for(j=0;list[j];j++)
       {
-       URL *stylesheetUrl=SplitURL(list[j]);
+       URL *stylesheetUrl=list[j];
 
        recurse_stylesheets--;
 
-       if(!stylesheetUrl->local && stylesheetUrl->Protocol)
+       if(!IsLocalHost(stylesheetUrl) && IsProtocolHandled(stylesheetUrl))
          {
           PrintMessage(Debug,"Style-Sheet=%s",stylesheetUrl->name);
           more+=request_url(stylesheetUrl,NULL,Url);
          }
 
        recurse_stylesheets++;
-
-       FreeURL(stylesheetUrl);
       }
 
  /* Any images. */
@@ -396,14 +426,14 @@ int RecurseFetch(URL *Url)
  if(recurse_images && (list=GetReferences(RefImage)))
     for(j=0;list[j];j++)
       {
-       URL *imageUrl=SplitURL(list[j]);
+       URL *imageUrl=list[j];
 
        recurse_images--;
 
-       if(!imageUrl->local && imageUrl->Protocol)
+       if(!IsLocalHost(imageUrl) && IsProtocolHandled(imageUrl))
          {
           if(!ConfigBooleanURL(FetchSameHostImages,Url) ||
-             (!strcmp(Url->proto,imageUrl->proto) && !strcmp(Url->host,imageUrl->host)))
+             (!strcmp(Url->proto,imageUrl->proto) && !strcmp(Url->hostport,imageUrl->hostport)))
             {
              PrintMessage(Debug,"Image=%s",imageUrl->name);
              more+=request_url(imageUrl,NULL,Url);
@@ -413,8 +443,6 @@ int RecurseFetch(URL *Url)
          }
 
        recurse_images++;
-
-       FreeURL(imageUrl);
       }
 
  /* Any frames */
@@ -422,11 +450,11 @@ int RecurseFetch(URL *Url)
  if(recurse_frames && (list=GetReferences(RefFrame)))
     for(j=0;list[j];j++)
       {
-       URL *frameUrl=SplitURL(list[j]);
+       URL *frameUrl=list[j];
 
        recurse_frames--;
 
-       if(!frameUrl->local && frameUrl->Protocol)
+       if(!IsLocalHost(frameUrl) && IsProtocolHandled(frameUrl))
          {
           char *refresh=NULL;
 
@@ -438,11 +466,11 @@ int RecurseFetch(URL *Url)
 
           PrintMessage(Debug,"Frame=%s",refresh?refresh:frameUrl->name);
           more+=request_url(frameUrl,refresh,Url);
+          if(refresh)
+             free(refresh);
          }
 
        recurse_frames++;
-
-       FreeURL(frameUrl);
       }
 
  /* Any scripts. */
@@ -450,19 +478,17 @@ int RecurseFetch(URL *Url)
  if(recurse_scripts && (list=GetReferences(RefScript)))
     for(j=0;list[j];j++)
       {
-       URL *scriptUrl=SplitURL(list[j]);
+       URL *scriptUrl=list[j];
 
        recurse_scripts--;
 
-       if(!scriptUrl->local && scriptUrl->Protocol)
+       if(!IsLocalHost(scriptUrl) && IsProtocolHandled(scriptUrl))
          {
           PrintMessage(Debug,"Script=%s",scriptUrl->name);
           more+=request_url(scriptUrl,NULL,Url);
          }
 
        recurse_scripts++;
-
-       FreeURL(scriptUrl);
       }
 
  /* Any Objects. */
@@ -470,29 +496,27 @@ int RecurseFetch(URL *Url)
  if(recurse_objects && (list=GetReferences(RefObject)))
     for(j=0;list[j];j++)
       {
-       URL *objectUrl=SplitURL(list[j]);
+       URL *objectUrl=list[j];
 
        recurse_objects--;
 
-       if(!objectUrl->local && objectUrl->Protocol)
+       if(!IsLocalHost(objectUrl) && IsProtocolHandled(objectUrl))
          {
           PrintMessage(Debug,"Object=%s",objectUrl->name);
           more+=request_url(objectUrl,NULL,Url);
          }
 
        recurse_objects++;
-
-       FreeURL(objectUrl);
       }
 
  if(recurse_objects && (list=GetReferences(RefInlineObject)))
     for(j=0;list[j];j++)
       {
-       URL *objectUrl=SplitURL(list[j]);
+       URL *objectUrl=list[j];
 
        recurse_objects--;
 
-       if(!objectUrl->local && objectUrl->Protocol)
+       if(!IsLocalHost(objectUrl) && IsProtocolHandled(objectUrl))
          {
           char *refresh=NULL;
 
@@ -504,11 +528,11 @@ int RecurseFetch(URL *Url)
 
           PrintMessage(Debug,"InlineObject=%s",refresh?refresh:objectUrl->name);
           more+=request_url(objectUrl,refresh,Url);
+          if(refresh)
+             free(refresh);
          }
 
        recurse_objects++;
-
-       FreeURL(objectUrl);
       }
 
  /* Any links */
@@ -516,11 +540,11 @@ int RecurseFetch(URL *Url)
  if(recurse_depth>0 && (list=GetReferences(RefLink)))
     for(j=0;list[j];j++)
       {
-       URL *linkUrl=SplitURL(list[j]);
+       URL *linkUrl=list[j];
 
        recurse_depth--;
 
-       if(!linkUrl->local && linkUrl->Protocol)
+       if(!IsLocalHost(linkUrl) && IsProtocolHandled(linkUrl))
           if(!*recurse_limit || !strncmp(recurse_limit,linkUrl->name,strlen(recurse_limit)))
             {
              char *refresh=NULL;
@@ -531,11 +555,11 @@ int RecurseFetch(URL *Url)
 
              PrintMessage(Debug,"Link=%s",refresh);
              more+=request_url(linkUrl,refresh,Url);
+             if(refresh)
+                free(refresh);
             }
 
        recurse_depth++;
-
-       FreeURL(linkUrl);
       }
 
  recurse_location=old_recurse_location;
@@ -551,15 +575,14 @@ int RecurseFetch(URL *Url)
 
   URL *Url The URL that was fetched.
 
-  char *location The new location of the URL.
+  URL *locationUrl The new location of the URL.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int RecurseFetchRelocation(URL *Url,char *location)
+int RecurseFetchRelocation(URL *Url,URL *locationUrl)
 {
  int more=0;
- URL *locationUrl=SplitURL(location);
 
- if(recurse_location && !locationUrl->local && locationUrl->Protocol)
+ if(recurse_location && !IsLocalHost(locationUrl) && IsProtocolHandled(locationUrl))
    {
     char *refresh=NULL;
 
@@ -573,6 +596,8 @@ int RecurseFetchRelocation(URL *Url,char *location)
 
     PrintMessage(Debug,"Location=%s",refresh?refresh:locationUrl->name);
     more+=request_url(locationUrl,refresh,Url);
+    if(refresh)
+       free(refresh);
 
     recurse_location++;
    }
@@ -603,7 +628,7 @@ static int request_url(URL *Url,char *refresh,URL *refUrl)
     PrintMessage(Debug,"The URL '%s' matches one in the list not to get.",Url->name);
  else
    {
-    int new_outgoing=OpenOutgoingSpoolFile(0);
+    int new_outgoing=OpenNewOutgoingSpoolFile();
 
     if(new_outgoing==-1)
        PrintMessage(Warning,"Cannot open the new outgoing request to write.");
@@ -611,19 +636,19 @@ static int request_url(URL *Url,char *refresh,URL *refUrl)
       {
        URL *reqUrl;
        Header *new_request_head;
-       char *head,str[32];
+       char *head,str[MAX_INT_STR+24];
 
        init_io(new_outgoing);
 
-       if(refUrl->pass && !strcmp(refUrl->host,Url->host))
-          AddURLPassword(Url,refUrl->user,refUrl->pass);
+       if(refUrl->pass && !strcmp(refUrl->hostport,Url->hostport))
+          AddPasswordURL(Url,refUrl->user,refUrl->pass);
 
        if(refresh)
           reqUrl=SplitURL(refresh);
        else
           reqUrl=Url;
 
-       new_request_head=RequestURL(reqUrl,refUrl->name);
+       new_request_head=RequestURL(reqUrl,refUrl);
 
        if(recurse_force)
           AddToHeader(new_request_head,"Pragma","no-cache");
@@ -651,7 +676,7 @@ static int request_url(URL *Url,char *refresh,URL *refUrl)
           PrintMessage(Warning,"Cannot write to outgoing file; disk full?");
 
        finish_io(new_outgoing);
-       CloseOutgoingSpoolFile(new_outgoing,reqUrl);
+       CloseNewOutgoingSpoolFile(new_outgoing,reqUrl);
 
        retval=1;
 
@@ -669,7 +694,7 @@ static int request_url(URL *Url,char *refresh,URL *refUrl)
 /*++++++++++++++++++++++++++++++++++++++
   Create the special path with arguments for doing a refresh with options.
 
-  char *CreateRefreshPath Returns a new string.
+  char *CreateRefreshPath Returns a new allocated string.
 
   URL *Url The URL that is to be fetched (starting point).
 
