@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/src/RCS/wwwoffled.c 2.66 2003/10/19 15:40:45 amb Exp $
+  $Header: /home/amb/wwwoffle/src/RCS/wwwoffled.c 2.81 2005/12/11 10:06:20 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.8.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.9.
   A demon program to maintain the database and spawn the servers.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1996,97,98,99,2000,01,02,03 Andrew M. Bishop
+  This file Copyright 1996,97,98,99,2000,01,02,03,04,05 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -47,6 +47,7 @@
 #include "errors.h"
 #include "config.h"
 #include "sockets.h"
+#include "certificates.h"
 #include "version.h"
 
 
@@ -60,6 +61,7 @@ static void sighup(int signum);
 
 /*+ The server sockets that we listen on +*/
 int http_fd[2]={-1,-1},          /*+ for the HTTP connections. +*/
+    https_fd[2]={-1,-1},         /*+ for the HTTPS connections. +*/
     wwwoffle_fd[2]={-1,-1};      /*+ for the WWWOFFLE connections. +*/
 
 /*+ The online / offline /autodial status. +*/
@@ -91,11 +93,11 @@ int purge_pid=0;
 /*+ The current status, fetching or not. +*/
 int fetching=0;
 
-/*+ Set to 1 when the demon is to shutdown. +*/
-int closedown=0;
+/*+ Set when the demon is to shutdown due to a sigexit. +*/
+static sig_atomic_t got_sigexit=0;
 
-/*+ Set to 1 when the demon is to re-read config file due to a sighup. +*/
-static int readconfig=0;
+/*+ Set when the demon is to re-read config file due to a sighup. +*/
+static sig_atomic_t got_sighup=0;
 
 /*+ Remember that we got a SIGCHLD +*/
 static sig_atomic_t got_sigchld=0;
@@ -109,9 +111,6 @@ static int print_pid=0;
 /*+ True if the -f option was passed on the command line. +*/
 int nofork=0;
 
-/* Global variables used to pass on information about remote client to CGI. */
-extern char *client_hostname,*client_ip;
-
 
 /*++++++++++++++++++++++++++++++++++++++
   The main program.
@@ -122,7 +121,7 @@ int main(int argc, char** argv)
  int i;
  int err;
  struct stat buf;
- char *config_file=NULL;
+ char *config_file=NULL,*log_file=NULL;
  int uid,gid;
 
  /* Parse the command line options */
@@ -141,15 +140,26 @@ int main(int argc, char** argv)
 
        if(i<(argc-1) && isdigit(argv[i+1][0]))
          {
-          DebuggingLevel=Fatal+1-atoi(argv[++i]);
+          StderrLevel=Fatal+1-atoi(argv[++i]);
 
-          if(DebuggingLevel<0 || DebuggingLevel>Fatal)
-            {fprintf(stderr,"wwwoffled: The '-d' option requires a number between 1 and %d.\n",Fatal+1); exit(1);}
+          if(StderrLevel<0 || StderrLevel>Fatal)
+            {fprintf(stderr,"wwwoffled: The '-d' option requires a number between 0 and %d.\n",Fatal+1); exit(1);}
          }
 
        continue;
       }
 
+    if(!strcmp(argv[i],"-l"))
+      {
+       if(++i>=argc)
+         {fprintf(stderr,"wwwoffled: The '-l' option requires a filename.\n"); exit(1);}
+       if(argv[i][0]!='/')
+         {fprintf(stderr,"wwwoffled: The '-l' option requires an absolute pathname.\n"); exit(1);}
+
+       log_file=argv[i];
+       continue;
+      }
+
     if(!strcmp(argv[i],"-p"))
       {
        print_pid=1;
@@ -175,6 +185,19 @@ int main(int argc, char** argv)
     fprintf(stderr,"wwwoffled: Unknown option '%s'.\n",argv[i]); exit(1);
    }
 
+ /* Combination options */
+
+ if(log_file)
+   {
+    if(nofork)
+       log_file=NULL;           /* -f overrides -l */
+    else
+       detached=1;              /* -l overrides -d */
+
+    if(StderrLevel==-1)
+       StderrLevel=Inform;      /* -l sets log level if no -d */
+   }
+
  /* Initialise things. */
 
  for(i=0;i<MAX_FETCH_SERVERS;i++)
@@ -183,36 +206,45 @@ int main(int argc, char** argv)
  for(i=0;i<MAX_SERVERS;i++)
     server_pids[i]=0;
 
+ if(log_file)
+    OpenErrorLog(log_file);
+
  InitErrorHandler("wwwoffled",0,1); /* use stderr and not syslog to start with. */
 
  /* Read the configuration file. */
 
  InitConfigurationFile(config_file);
 
- init_io(2);
+ init_io(STDERR_FILENO);
 
- if(ReadConfigurationFile(2))
+ if(ReadConfigurationFile(STDERR_FILENO))
     PrintMessage(Fatal,"Error in configuration file '%s'.",ConfigurationFileName());
 
- finish_io(2);
+ finish_io(STDERR_FILENO);
 
  InitErrorHandler("wwwoffled",ConfigInteger(UseSyslog),1); /* enable syslog if requested. */
 
  /* Print a startup message. */
 
-#if USE_ZLIB
-#define ZLIB_STRING "with zlib"
-#else
-#define ZLIB_STRING "without zlib"
-#endif
-
 #if USE_IPV6
 #define IPV6_STRING "with ipv6"
 #else
 #define IPV6_STRING "without ipv6"
 #endif
 
- PrintMessage(Important,"WWWOFFLE Demon Version %s (%s,%s) started.",WWWOFFLE_VERSION,ZLIB_STRING,IPV6_STRING);
+#if USE_ZLIB
+#define ZLIB_STRING "with zlib"
+#else
+#define ZLIB_STRING "without zlib"
+#endif
+
+#if USE_GNUTLS
+#define GNUTLS_STRING "with gnutls"
+#else
+#define GNUTLS_STRING "without gnutls"
+#endif
+
+ PrintMessage(Important,"WWWOFFLE Demon Version %s (%s,%s,%s) started.",WWWOFFLE_VERSION,IPV6_STRING,ZLIB_STRING,GNUTLS_STRING);
  PrintMessage(Inform,"WWWOFFLE Read Configuration File '%s'.",ConfigurationFileName());
 
  /* Change the user and group. */
@@ -223,29 +255,32 @@ int main(int argc, char** argv)
  if(uid!=-1)
     seteuid(0);
 
+ if(log_file && (uid!=-1 || gid!=-1))
+    chown(log_file,(uid_t)uid,(gid_t)gid);
+
  if(gid!=-1)
    {
 #if HAVE_SETGROUPS
     if(getuid()==0 || geteuid()==0)
-       if(setgroups(0, NULL)<0)
+       if(setgroups(0,NULL)<0)
           PrintMessage(Fatal,"Cannot clear supplementary group list [%!s].");
 #endif
 
 #if HAVE_SETRESGID
-    if(setresgid(gid,gid,gid)<0)
+    if(setresgid((gid_t)gid,(gid_t)gid,(gid_t)gid)<0)
        PrintMessage(Fatal,"Cannot set real/effective/saved group id to %d [%!s].",gid);
 #else
     if(geteuid()==0)
       {
-       if(setgid(gid)<0)
+       if(setgid((gid_t)gid)<0)
           PrintMessage(Fatal,"Cannot set group id to %d [%!s].",gid);
       }
     else
       {
 #if HAVE_SETREGID
-       if(setregid(getegid(),gid)<0)
+       if(setregid(getegid(),(gid_t)gid)<0)
           PrintMessage(Fatal,"Cannot set effective group id to %d [%!s].",gid);
-       if(setregid(gid,-1)<0)
+       if(setregid((gid_t)gid,(gid_t)~1)<0)
           PrintMessage(Fatal,"Cannot set real group id to %d [%!s].",gid);
 #else
        PrintMessage(Fatal,"Must be root to totally change group id.");
@@ -257,20 +292,20 @@ int main(int argc, char** argv)
  if(uid!=-1)
    {
 #if HAVE_SETRESUID
-    if(setresuid(uid,uid,uid)<0)
+    if(setresuid((uid_t)uid,(uid_t)uid,(uid_t)uid)<0)
        PrintMessage(Fatal,"Cannot set real/effective/saved user id to %d [%!s].",uid);
 #else
     if(geteuid()==0)
       {
-       if(setuid(uid)<0)
+       if(setuid((uid_t)uid)<0)
           PrintMessage(Fatal,"Cannot set user id to %d [%!s].",uid);
       }
     else
       {
 #if HAVE_SETREUID
-       if(setreuid(geteuid(),uid)<0)
+       if(setreuid(geteuid(),(uid_t)uid)<0)
           PrintMessage(Fatal,"Cannot set effective user id to %d [%!s].",uid);
-       if(setreuid(uid,-1)<0)
+       if(setreuid((uid_t)uid,(uid_t)~1)<0)
           PrintMessage(Fatal,"Cannot set real user id to %d [%!s].",uid);
 #else
        PrintMessage(Fatal,"Must be root to totally change user id.");
@@ -285,80 +320,6 @@ int main(int argc, char** argv)
  if(geteuid()==0 || getegid()==0)
     PrintMessage(Warning,"Running with root user or group privileges is not recommended.");
 
- /* Bind the HTTP proxy socket(s). */
-
-#if USE_IPV6
- if(ConfigString(Bind_IPv6))
-   {
-    http_fd[1]=OpenServerSocket(ConfigString(Bind_IPv6),ConfigInteger(HTTP_Port));
-    if(http_fd[1]==-1)
-       PrintMessage(Fatal,"Cannot create HTTP IPv6 server socket.");
-   }
-#endif
-
- if(ConfigString(Bind_IPv4))
-   {
-    http_fd[0]=OpenServerSocket(ConfigString(Bind_IPv4),ConfigInteger(HTTP_Port));
-    if(http_fd[0]==-1)
-      {
-#if USE_IPV6
-       if(http_fd[1]!=-1 && ConfigString(Bind_IPv4) && !strcmp(ConfigString(Bind_IPv4),"0.0.0.0") &&
-                            ConfigString(Bind_IPv6) && !strcmp(ConfigString(Bind_IPv6),"0:0:0:0:0:0:0:0"))
-          PrintMessage(Warning,"Cannot create HTTP IPv4 server socket (but the IPv6 one might accept IPv4 connections).");
-       else
-          PrintMessage(Fatal,"Cannot create HTTP IPv4 server socket.");
-#else
-       PrintMessage(Fatal,"Cannot create HTTP server socket.");
-#endif
-      }
-   }
-
- if(http_fd[0]==-1 && http_fd[1]==-1)
-   {
-#if USE_IPV6
-    PrintMessage(Fatal,"The IPv4 and IPv6 HTTP sockets were not bound; are they disabled in the config file?");
-#else
-    PrintMessage(Fatal,"The HTTP socket was not bound; are they disabled in the config file?");
-#endif
-   }
-
- /* Bind the WWWOFFLE control socket(s). */
-
-#if USE_IPV6
- if(ConfigString(Bind_IPv6))
-   {
-    wwwoffle_fd[1]=OpenServerSocket(ConfigString(Bind_IPv6),ConfigInteger(WWWOFFLE_Port));
-    if(wwwoffle_fd[1]==-1)
-       PrintMessage(Fatal,"Cannot create WWWOFFLE IPv6 server socket.");
-   }
-#endif
-
- if(ConfigString(Bind_IPv4))
-   {
-    wwwoffle_fd[0]=OpenServerSocket(ConfigString(Bind_IPv4),ConfigInteger(WWWOFFLE_Port));
-    if(wwwoffle_fd[0]==-1)
-      {
-#if USE_IPV6
-       if(wwwoffle_fd[1]!=-1 && ConfigString(Bind_IPv4) && !strcmp(ConfigString(Bind_IPv4),"0.0.0.0") &&
-                                ConfigString(Bind_IPv6) && !strcmp(ConfigString(Bind_IPv6),"0:0:0:0:0:0:0:0"))
-          PrintMessage(Warning,"Cannot create WWWOFFLE IPv4 server socket (but the IPv6 one might accept IPv4 connections).");
-       else
-          PrintMessage(Fatal,"Cannot create WWWOFFLE IPv4 server socket.");
-#else
-       PrintMessage(Fatal,"Cannot create WWWOFFLE server socket.");
-#endif
-      }
-   }
-
- if(wwwoffle_fd[0]==-1 && wwwoffle_fd[1]==-1)
-   {
-#if USE_IPV6
-    PrintMessage(Fatal,"The IPv4 and IPv6 WWWOFFLE sockets were not bound; are they disabled in the config file?");
-#else
-    PrintMessage(Fatal,"The WWWOFFLE socket was not bound; are they disabled in the config file?");
-#endif
-   }
-
  /* Create, Change to and open the spool directory. */
 
  umask(0);
@@ -378,15 +339,151 @@ int main(int argc, char** argv)
  if(err==-1)
     PrintMessage(Fatal,"Cannot change to spool directory %s [%!s].",ConfigString(SpoolDir));
 
+ /* Bind the HTTP proxy socket(s). */
+
+#if USE_IPV6
+ if(ConfigString(Bind_IPv6))
+   {
+    http_fd[1]=OpenServerSocket(ConfigString(Bind_IPv6),ConfigInteger(HTTP_Port));
+    if(http_fd[1]==-1)
+       PrintMessage(Fatal,"Cannot create HTTP IPv6 server socket.");
+   }
+#endif
+
+ if(ConfigString(Bind_IPv4))
+   {
+    http_fd[0]=OpenServerSocket(ConfigString(Bind_IPv4),ConfigInteger(HTTP_Port));
+    if(http_fd[0]==-1)
+      {
+#if USE_IPV6
+       if(http_fd[1]!=-1 && ConfigString(Bind_IPv4) && !strcmp(ConfigString(Bind_IPv4),"0.0.0.0") &&
+                            ConfigString(Bind_IPv6) && !strcmp(ConfigString(Bind_IPv6),"[0:0:0:0:0:0:0:0]"))
+          PrintMessage(Warning,"Cannot create HTTP IPv4 server socket (but the IPv6 one might accept IPv4 connections).");
+       else
+          PrintMessage(Fatal,"Cannot create HTTP IPv4 server socket.");
+#else
+       PrintMessage(Fatal,"Cannot create HTTP server socket.");
+#endif
+      }
+   }
+
+ if(http_fd[0]==-1 && http_fd[1]==-1)
+   {
+#if USE_IPV6
+    PrintMessage(Fatal,"The IPv4 and IPv6 HTTP sockets were not bound; are they disabled in the config file?");
+#else
+    PrintMessage(Fatal,"The HTTP socket was not bound; is it disabled in the config file?");
+#endif
+   }
+
+ /* Bind the HTTPS socket(s). */
+
+#if USE_GNUTLS
+
+ if(LoadRootCredentials())
+   {
+    PrintMessage(Warning,"Failed to read (or create if needed) the WWWOFFLE root certificates.");
+   }
+
+ if(LoadTrustedCertificates())
+   {
+    PrintMessage(Warning,"Failed to read in any trusted certificates.");
+   }
+
+#if USE_IPV6
+ if(ConfigString(Bind_IPv6))
+   {
+    https_fd[1]=OpenServerSocket(ConfigString(Bind_IPv6),ConfigInteger(HTTPS_Port));
+    if(https_fd[1]==-1)
+       PrintMessage(Fatal,"Cannot create HTTPS IPv6 server socket.");
+   }
+#endif
+
+ if(ConfigString(Bind_IPv4))
+   {
+    https_fd[0]=OpenServerSocket(ConfigString(Bind_IPv4),ConfigInteger(HTTPS_Port));
+    if(https_fd[0]==-1)
+      {
+#if USE_IPV6
+       if(https_fd[1]!=-1 && ConfigString(Bind_IPv4) && !strcmp(ConfigString(Bind_IPv4),"0.0.0.0") &&
+                             ConfigString(Bind_IPv6) && !strcmp(ConfigString(Bind_IPv6),"[0:0:0:0:0:0:0:0]"))
+          PrintMessage(Warning,"Cannot create HTTPS IPv4 server socket (but the IPv6 one might accept IPv4 connections).");
+       else
+          PrintMessage(Fatal,"Cannot create HTTPS IPv4 server socket.");
+#else
+       PrintMessage(Fatal,"Cannot create HTTPS server socket.");
+#endif
+      }
+   }
+
+ if(https_fd[0]==-1 && https_fd[1]==-1)
+   {
+#if USE_IPV6
+    PrintMessage(Fatal,"The IPv4 and IPv6 HTTPS sockets were not bound; are they disabled in the config file?");
+#else
+    PrintMessage(Fatal,"The HTTPS socket was not bound; is it disabled in the config file?");
+#endif
+   }
+
+#endif /* USE_GNUTLS */
+
+ /* Bind the WWWOFFLE control socket(s). */
+
+#if USE_IPV6
+ if(ConfigString(Bind_IPv6))
+   {
+    wwwoffle_fd[1]=OpenServerSocket(ConfigString(Bind_IPv6),ConfigInteger(WWWOFFLE_Port));
+    if(wwwoffle_fd[1]==-1)
+       PrintMessage(Fatal,"Cannot create WWWOFFLE IPv6 server socket.");
+   }
+#endif
+
+ if(ConfigString(Bind_IPv4))
+   {
+    wwwoffle_fd[0]=OpenServerSocket(ConfigString(Bind_IPv4),ConfigInteger(WWWOFFLE_Port));
+    if(wwwoffle_fd[0]==-1)
+      {
+#if USE_IPV6
+       if(wwwoffle_fd[1]!=-1 && ConfigString(Bind_IPv4) && !strcmp(ConfigString(Bind_IPv4),"0.0.0.0") &&
+                                ConfigString(Bind_IPv6) && !strcmp(ConfigString(Bind_IPv6),"[0:0:0:0:0:0:0:0]"))
+          PrintMessage(Warning,"Cannot create WWWOFFLE IPv4 server socket (but the IPv6 one might accept IPv4 connections).");
+       else
+          PrintMessage(Fatal,"Cannot create WWWOFFLE IPv4 server socket.");
+#else
+       PrintMessage(Fatal,"Cannot create WWWOFFLE server socket.");
+#endif
+      }
+   }
+
+ if(wwwoffle_fd[0]==-1 && wwwoffle_fd[1]==-1)
+   {
+#if USE_IPV6
+    PrintMessage(Fatal,"The IPv4 and IPv6 WWWOFFLE sockets were not bound; are they disabled in the config file?");
+#else
+    PrintMessage(Fatal,"The WWWOFFLE socket was not bound; is it disabled in the config file?");
+#endif
+   }
+
  /* Detach from terminal */
 
  if(detached)
    {
     demoninit();
+
     PrintMessage(Important,"Detached from terminal and changed pid to %d.",getpid());
-    InitErrorHandler("wwwoffled",1,0); /* pid changes after detaching, enable syslog, disable stderr. */
+
+    if(log_file)
+       InitErrorHandler("wwwoffled",-1,-1); /* pid changes after detaching, keep stderr as was. */
+    else
+       InitErrorHandler("wwwoffled",-1, 0); /* pid changes after detaching, disable stderr. */
+
+    if(!log_file)
+       close(STDERR_FILENO);
    }
 
+ close(STDIN_FILENO);
+ close(STDOUT_FILENO);
+
  install_sighandlers();
 
  max_servers=ConfigInteger(MaxServers);
@@ -408,12 +505,18 @@ int main(int argc, char** argv)
 #endif
           if(http_fd[nfd]>=nfds)
              nfds=http_fd[nfd]+1;
+          if(https_fd[nfd]>=nfds)
+             nfds=https_fd[nfd]+1;
           if(wwwoffle_fd[nfd]>=nfds)
              nfds=wwwoffle_fd[nfd]+1;
 
           if(n_servers<max_servers)
+            {
              if(http_fd[nfd]!=-1)
                 FD_SET(http_fd[nfd],&readfd);
+             if(https_fd[nfd]!=-1)
+                FD_SET(https_fd[nfd],&readfd);
+            }
 
           if(wwwoffle_fd[nfd]!=-1)
              FD_SET(wwwoffle_fd[nfd],&readfd);
@@ -440,8 +543,7 @@ int main(int argc, char** argv)
              if(client>=0)
                {
                 init_io(client);
-                configure_io_read(client,ConfigInteger(SocketTimeout),0,0);
-                configure_io_write(client,ConfigInteger(SocketTimeout),0,0);
+                configure_io_timeout(client,ConfigInteger(SocketTimeout),ConfigInteger(SocketTimeout));
 
                 if(SocketRemoteName(client,&host,&ip,&port))
                   {
@@ -471,8 +573,7 @@ int main(int argc, char** argv)
                       CloseSocket(client);
                      }
 
-                   if(canonical_ip!=ip)
-                      free(canonical_ip);
+                   free(canonical_ip);
                   }
                }
             }
@@ -487,8 +588,7 @@ int main(int argc, char** argv)
              if(client>=0)
                {
                 init_io(client);
-                configure_io_read(client,ConfigInteger(SocketTimeout),0,0);
-                configure_io_write(client,ConfigInteger(SocketTimeout),0,0);
+                configure_io_timeout(client,ConfigInteger(SocketTimeout),ConfigInteger(SocketTimeout));
 
                 if(!SocketRemoteName(client,&host,&ip,&port))
                   {
@@ -497,21 +597,16 @@ int main(int argc, char** argv)
                    if(IsAllowedConnectHost(host) || IsAllowedConnectHost(canonical_ip))
                      {
                       PrintMessage(Inform,"HTTP Proxy connection from host %s (%s).",host,canonical_ip); /* Used in audit-usage.pl */
-
-                      client_hostname=host;
-                      client_ip=canonical_ip;
-
                       ForkServer(client);
                      }
                    else
                       PrintMessage(Warning,"HTTP Proxy connection rejected from host %s (%s).",host,canonical_ip); /* Used in audit-usage.pl */
 
-                   if(canonical_ip!=ip)
-                      free(canonical_ip);
+                   free(canonical_ip);
                   }
 
                 if(nofork)
-                   closedown=1;
+                   got_sigexit=1;
 
                 if(!nofork)
                   {
@@ -520,17 +615,65 @@ int main(int argc, char** argv)
                   }
                }
             }
+
+#if USE_GNUTLS
+          if(https_fd[nfd]!=-1 && FD_ISSET(https_fd[nfd],&readfd))
+            {
+             char *host,*ip;
+             int port,client;
+
+             client=AcceptConnect(https_fd[nfd]);
+
+             if(client>=0)
+               {
+                init_io(client);
+                configure_io_timeout(client,ConfigInteger(SocketTimeout),ConfigInteger(SocketTimeout));
+
+                if(!SocketRemoteName(client,&host,&ip,&port))
+                  {
+                   char *canonical_ip=CanonicaliseHost(ip);
+
+                   if(IsAllowedConnectHost(host) || IsAllowedConnectHost(canonical_ip))
+                     {
+                      PrintMessage(Inform,"HTTPS Proxy connection from host %s (%s).",host,canonical_ip); /* Used in audit-usage.pl */
+                      ForkServer(client);
+                     }
+                   else
+                      PrintMessage(Warning,"HTTPS Proxy connection rejected from host %s (%s).",host,canonical_ip); /* Used in audit-usage.pl */
+
+                   free(canonical_ip);
+                  }
+
+                if(nofork)
+                   got_sigexit=1;
+
+                if(!nofork)
+                  {
+                   finish_io(client);
+                   CloseSocket(client);
+                  }
+               }
+            }
+#endif /* USE_GNUTLS */
+
 #if USE_IPV6
          }
 #endif
       }
 
-    if(readconfig)
+    if(got_sighup)
       {
-       readconfig=0;
+       got_sighup=0;
 
        PrintMessage(Important,"SIGHUP signalled.");
 
+       if(log_file)
+         {
+          PrintMessage(Important,"Closing and opening log file.");
+
+          OpenErrorLog(log_file);
+         }
+
        PrintMessage(Important,"WWWOFFLE Re-reading Configuration File.");
 
        if(ReadConfigurationFile(-1))
@@ -597,10 +740,10 @@ int main(int argc, char** argv)
           if(exitval==3)
              fetching=0;
 
-          if(exitval==4 && online==1)
+          if(exitval==4 && online!=0)
              fetching=1;
 
-          if(online!=1)
+          if(online==0)
              fetching=0;
          }
 
@@ -627,7 +770,7 @@ int main(int argc, char** argv)
        ForkRunModeScript(ConfigString(RunFetch),"fetch","stop",-1);
       }
    }
- while(!closedown);
+ while(!got_sigexit);
 
  /* Close down and exit. */
 
@@ -690,6 +833,10 @@ int main(int argc, char** argv)
 
  FinishConfigurationFile();
 
+#if USE_GNUTLS
+ FreeLoadedCredentials();
+#endif
+
  return(0);
 }
 
@@ -723,25 +870,28 @@ static void usage(int verbose)
 
  fprintf(stderr,
          "Usage: wwwoffled -h | --help | --version\n"
-         "       wwwoffled [-c <config-file>] [-d [<log-level>]]\n"
+         "       wwwoffled [-c <config-file>] [-d [<log-level>]] [-l <log-file>] [-f] [-p]\n"
          "\n");
 
  if(verbose)
     fprintf(stderr,
-            "   -h | --help         : Display this help.\n"
+            "   -h | --help      : Display this help.\n"
             "\n"
-            "   --version           : Display the version number of wwwoffled.\n"
+            "   --version        : Display the version number of wwwoffled.\n"
             "\n"
-            "   -c <config-file>    : The name of the configuration file to use.\n"
-            "                         (Default %s).\n"
+            "   -c <config-file> : The name of the configuration file to use.\n"
+            "                      (Default %s).\n"
             "\n"
-            "   -d [<log-level>]    : Do not detach from the terminal and use stderr.\n"
-            "                         0 <= log-level <= %d (default in config file).\n"
+            "   -d [<log-level>] : Do not detach from the terminal and use stderr.\n"
+            "                      0 <= log-level <= %d (default in config file).\n"
             "\n"
-            "   -f                  : Do not fork children for each HTTP request.\n"
-            "                         (Exits after first request, implies -d)\n"
+            "   -l <log-file>    : Detach from the terminal and use specified log file.\n"
+            "                      (Using log-level = 4 unless -d specifies different.)\n"
             "\n"
-            "   -p                  : Print the pid of wwwoffled on stdout (unless -d).\n"
+            "   -f               : Do not fork children for each HTTP request.\n"
+            "                      (Exits after first request, implies -d, overrides -l.)\n"
+            "\n"
+            "   -p               : Print the pid of wwwoffled on stdout (unless -d).\n"
             "\n",ConfigurationFileName(),Fatal+1);
 
  exit(0);
@@ -775,11 +925,6 @@ static void demoninit(void)
 
     exit(0);
    }
-
- /* Close fds */
- close(STDIN_FILENO);
- close(STDOUT_FILENO);
- close(STDERR_FILENO);
 }
 
 
@@ -793,7 +938,7 @@ static void install_sighandlers(void)
 
  /* SIGCHLD */
  action.sa_handler = sigchild;
- sigemptyset (&action.sa_mask);
+ sigemptyset(&action.sa_mask);
  action.sa_flags = 0;
  if(sigaction(SIGCHLD, &action, NULL) != 0)
     PrintMessage(Warning, "Cannot install SIGCHLD handler.");
@@ -821,7 +966,7 @@ static void install_sighandlers(void)
 
  /* SIGPIPE */
  action.sa_handler = SIG_IGN;
- sigemptyset (&action.sa_mask);
+ sigemptyset(&action.sa_mask);
  action.sa_flags = 0;
  if(sigaction(SIGPIPE, &action, NULL) != 0)
     PrintMessage(Warning, "Cannot ignore SIGPIPE.");
@@ -848,7 +993,7 @@ static void sigchild(/*@unused@*/ int signum)
 
 static void sigexit(/*@unused@*/ int signum)
 {
- closedown=1;
+ got_sigexit=1;
 }
 
 
@@ -860,5 +1005,5 @@ static void sigexit(/*@unused@*/ int signum)
 
 static void sighup(/*@unused@*/ int signum)
 {
- readconfig=1;
+ got_sighup=1;
 }
